@@ -113,20 +113,26 @@ class Pattern(Enum):
     DefaultUnprefixed = "default-unprefixed"
     """Default pattern, but without `v` prefix."""
 
-    def regex(self) -> str:
+    def regex(self, prefix: Optional[str] = None) -> str:
         """
         Get the regular expression for this preset pattern.
 
+        :param prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Regular expression.
         """
         variants = {
             Pattern.Default: VERSION_SOURCE_PATTERN,
             Pattern.DefaultUnprefixed: VERSION_SOURCE_PATTERN.replace("^v", "^v?", 1),
         }
-        return variants[self]
+
+        out = variants[self]
+        if prefix:
+            out = out.replace("^", "^{}".format(prefix), 1)
+
+        return out
 
     @staticmethod
-    def parse(pattern: Union[str, "Pattern"]) -> str:
+    def parse(pattern: Union[str, "Pattern"], prefix: Optional[str] = None) -> str:
         """
         Parse a pattern string into a regular expression.
 
@@ -135,10 +141,14 @@ class Pattern(Enum):
         `Pattern` enum.
 
         :param pattern: Pattern to parse.
+        :param prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Regular expression.
         """
         if isinstance(pattern, str) and "?P<base>" in pattern:
-            return pattern
+            if prefix:
+                return pattern.replace("^", "^{}".format(prefix), 1)
+            else:
+                return pattern
 
         try:
             pattern = Pattern(pattern)
@@ -152,7 +162,7 @@ class Pattern(Enum):
                     pattern,
                 )
             )
-        return pattern.regex()
+        return pattern.regex(prefix)
 
 
 class Concern(Enum):
@@ -226,8 +236,12 @@ _MatchedVersionPattern = NamedTuple(
 
 
 def _match_version_pattern(
-    pattern: Union[str, Pattern], sources: Sequence[str], latest_source: bool
-) -> _MatchedVersionPattern:
+    pattern: Union[str, Pattern],
+    sources: Sequence[str],
+    latest_source: bool,
+    strict: bool,
+    pattern_prefix: Optional[str],
+) -> Optional[_MatchedVersionPattern]:
     """
     :returns: Tuple of:
         * matched tag
@@ -245,7 +259,7 @@ def _match_version_pattern(
     tagged_metadata = None
     epoch = None  # type: Optional[Union[str, int]]
 
-    pattern = Pattern.parse(pattern)
+    pattern = Pattern.parse(pattern, pattern_prefix)
 
     for source in sources[:1] if latest_source else sources:
         try:
@@ -281,8 +295,10 @@ def _match_version_pattern(
                     sources,
                 )
             )
-        else:
+        elif strict:
             raise ValueError(_pattern_error("The pattern did not match any tags", pattern, sources))
+        else:
+            return None
 
     stage = pattern_match.groupdict().get("stage")
     revision = pattern_match.groupdict().get("revision")
@@ -327,6 +343,13 @@ def _get_git_version() -> List[int]:
     return []
 
 
+def _git_log(git_version: List[int]) -> str:
+    if git_version < [2, 10]:
+        return "git log"
+    else:
+        return "git -c log.showsignature=false log"
+
+
 def _detect_vcs(expected_vcs: Optional[Vcs], path: Optional[Path]) -> Vcs:
     checks = OrderedDict(
         [
@@ -480,9 +503,8 @@ class _GitRefInfo:
         tag_branch: str, git_version: List[int], path: Optional[Path]
     ) -> Mapping[str, int]:
         cmd = (
-            "git log --simplify-by-decoration --topo-order --decorate=full"
-            ' {} "--format=%H%d"'.format(tag_branch)
-        )
+            '{} --simplify-by-decoration --topo-order --decorate=full {} "--format=%H%d"'
+        ).format(_git_log(git_version), tag_branch)
         if git_version >= [2, 16]:
             cmd += " --decorate-refs=refs/tags/"
         code, logmsg = _run_cmd(cmd, path)
@@ -591,6 +613,7 @@ class Version:
 
         self._matched_tag = None  # type: Optional[str]
         self._newer_unmatched_tags = None  # type: Optional[Sequence[str]]
+        self._smart_bumped = False
 
     def __str__(self) -> str:
         return self.serialize()
@@ -723,8 +746,8 @@ class Version:
         """
         base = self.base
         revision = self.revision
-        if bump and self.distance > 0:
-            bumped = self.bump()
+        if bump:
+            bumped = self.bump(smart=True)
             base = bumped.base
             revision = bumped.revision
 
@@ -776,7 +799,7 @@ class Version:
             if revision is not None:
                 pre_parts.append(str(revision))
         if self.distance > 0:
-            pre_parts.append("pre" if bump else "post")
+            pre_parts.append("pre" if bump or self._smart_bumped else "post")
             pre_parts.append(str(self.distance))
 
         if style == Style.Pep440:
@@ -790,7 +813,7 @@ class Version:
                 stage = None
                 dev = revision
             if self.distance > 0:
-                if bump:
+                if bump or self._smart_bumped:
                     if dev is None:
                         dev = self.distance
                     else:
@@ -838,9 +861,15 @@ class Version:
         if not version.startswith("v") and pattern in [VERSION_SOURCE_PATTERN, Pattern.Default]:
             normalized = "v{}".format(version)
 
+        failed = False
         try:
-            matched_pattern = _match_version_pattern(pattern, [normalized], True)
+            matched_pattern = _match_version_pattern(
+                pattern, [normalized], True, strict=True, pattern_prefix=None
+            )
         except ValueError:
+            failed = True
+
+        if failed or matched_pattern is None:
             replaced = re.sub(r"(\.post(\d+)\.dev\d+)", r".dev\2", version, 1)
             if replaced != version:
                 alt = Version.parse(replaced, pattern)
@@ -908,7 +937,7 @@ class Version:
             epoch=epoch,
         )
 
-    def bump(self, index: int = -1, increment: int = 1) -> "Version":
+    def bump(self, index: int = -1, increment: int = 1, smart: bool = False) -> "Version":
         """
         Increment the version.
 
@@ -921,9 +950,18 @@ class Version:
             the right side and count down from -1.
             Only has an effect when the base is bumped.
         :param increment: By how much to increment the relevant position.
+        :param smart: If true, only bump when distance is not 0.
+            This will also make `Version.serialize()` use pre-release formatting automatically,
+            like calling `Version.serialize(bump=True)`.
         :returns: Bumped version.
         """
         bumped = copy.deepcopy(self)
+
+        if smart:
+            if bumped.distance == 0:
+                return bumped
+            bumped._smart_bumped = True
+
         if bumped.stage is None:
             bumped.base = bump_version(bumped.base, index, increment)
         else:
@@ -976,6 +1014,8 @@ class Version:
         full_commit: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
+        ignore_untracked: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Git tags.
@@ -991,6 +1031,9 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
+        :param ignore_untracked:
+            Ignore untracked files when determining whether the repository is dirty.
         :returns: Detected version.
         """
         vcs = Vcs.Git
@@ -1044,9 +1087,21 @@ class Version:
                         vcs=vcs,
                     )
 
-                tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-                    pattern, [tag], latest_tag
+                matched_pattern = _match_version_pattern(
+                    pattern, [tag], latest_tag, strict, pattern_prefix
                 )
+                if matched_pattern is None:
+                    return cls._fallback(
+                        strict,
+                        distance=distance,
+                        commit=commit,
+                        dirty=dirty,
+                        branch=branch,
+                        timestamp=timestamp,
+                        vcs=vcs,
+                    )
+                tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
+
                 version = cls(
                     base,
                     stage=stage,
@@ -1089,7 +1144,9 @@ class Version:
             branch = msg
 
         code, msg = _run_cmd(
-            'git log -n 1 --format="format:{}"'.format("%H" if full_commit else "%h"),
+            '{} -n 1 --format="format:{}"'.format(
+                _git_log(git_version), "%H" if full_commit else "%h"
+            ),
             path,
             codes=[0, 128],
         )
@@ -1101,18 +1158,20 @@ class Version:
 
         timestamp = None
         if git_version < [2, 2]:
-            code, msg = _run_cmd('git log -n 1 --pretty=format:"%ci"', path)
+            code, msg = _run_cmd(
+                '{} -n 1 --pretty=format:"%ci"'.format(_git_log(git_version)), path
+            )
             timestamp = _parse_git_timestamp_iso(msg)
         else:
             code, msg = _run_cmd(
-                'git -c log.showsignature=false log -n 1 --pretty=format:"%cI"', path
+                '{} -n 1 --pretty=format:"%cI"'.format(_git_log(git_version)), path
             )
             timestamp = _parse_git_timestamp_iso_strict(msg)
 
         code, msg = _run_cmd("git describe --always --dirty", path)
         dirty = msg.endswith("-dirty")
 
-        if not dirty:
+        if not dirty and not ignore_untracked:
             code, msg = _run_cmd("git status --porcelain", path)
             if msg.strip() != "":
                 dirty = True
@@ -1138,8 +1197,8 @@ class Version:
                     vcs=vcs,
                 )
             tags = [line.replace("refs/tags/", "") for line in msg.splitlines()]
-            tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-                pattern, tags, latest_tag
+            matched_pattern = _match_version_pattern(
+                pattern, tags, latest_tag, strict, pattern_prefix
             )
         else:
             code, msg = _run_cmd(
@@ -1174,13 +1233,36 @@ class Version:
 
             for line in msg.strip().splitlines():
                 parts = line.split("@{")
+                if len(parts) != 5:
+                    continue
                 detailed_tags.append(_GitRefInfo(*parts).with_tag_topo_lookup(tag_topo_lookup))
 
             tags = [t.ref for t in sorted(detailed_tags, key=lambda x: x.sort_key, reverse=True)]
-            tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-                pattern, tags, latest_tag
+            matched_pattern = _match_version_pattern(
+                pattern, tags, latest_tag, strict, pattern_prefix
             )
 
+        if matched_pattern is None:
+            distance = 0
+
+            code, msg = _run_cmd("git rev-list --max-parents=0 HEAD", path)
+            if msg:
+                initial_commit = msg.splitlines()[0].strip()
+                code, msg = _run_cmd("git rev-list --count {}..HEAD".format(initial_commit), path)
+                distance = int(msg)
+
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                branch=branch,
+                timestamp=timestamp,
+                concerns=concerns,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
+
         code, msg = _run_cmd("git rev-list --count refs/tags/{}..HEAD".format(tag), path)
         distance = int(msg)
 
@@ -1209,6 +1291,7 @@ class Version:
         full_commit: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Mercurial tags.
@@ -1222,6 +1305,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Mercurial
@@ -1259,9 +1343,19 @@ class Version:
                         continue
                     all_tags.append(parts[1])
 
-            tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-                pattern, all_tags, latest_tag
+            matched_pattern = _match_version_pattern(
+                pattern, all_tags, latest_tag, strict, pattern_prefix
             )
+            if matched_pattern is None:
+                return cls._fallback(
+                    strict,
+                    distance=distance,
+                    commit=commit,
+                    branch=branch,
+                    vcs=vcs,
+                )
+            tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
+
             version = cls(
                 base,
                 stage=stage,
@@ -1314,9 +1408,19 @@ class Version:
                 vcs=vcs,
             )
         tags = [tag for tags in [line.split(":") for line in msg.splitlines()] for tag in tags]
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, tags, latest_tag
-        )
+
+        matched_pattern = _match_version_pattern(pattern, tags, latest_tag, strict, pattern_prefix)
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                branch=branch,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
 
         code, msg = _run_cmd('hg log -r "{0}::{1} - {0}" --template "."'.format(tag, commit), path)
         # The tag itself is in the list, so offset by 1.
@@ -1345,6 +1449,7 @@ class Version:
         latest_tag: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Darcs tags.
@@ -1357,6 +1462,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Darcs
@@ -1385,9 +1491,18 @@ class Version:
                 strict, distance=distance, commit=commit, dirty=dirty, timestamp=timestamp, vcs=vcs
             )
         tags = msg.splitlines()
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, tags, latest_tag
-        )
+
+        matched_pattern = _match_version_pattern(pattern, tags, latest_tag, strict, pattern_prefix)
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
 
         code, msg = _run_cmd("darcs log --from-tag {} --count".format(tag), path)
         # The tag itself is in the list, so offset by 1.
@@ -1416,6 +1531,7 @@ class Version:
         tag_dir: str = "tags",
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Subversion tags.
@@ -1429,6 +1545,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Subversion
@@ -1479,9 +1596,18 @@ class Version:
                     source = int(match.group(1))
                     tags_to_sources_revs[tag] = (source, rev)
         tags = sorted(tags_to_sources_revs, key=lambda x: tags_to_sources_revs[x], reverse=True)
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, tags, latest_tag
-        )
+
+        matched_pattern = _match_version_pattern(pattern, tags, latest_tag, strict, pattern_prefix)
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
 
         source, rev = tags_to_sources_revs[tag]
         # The tag itself is in the list, so offset by 1.
@@ -1509,6 +1635,7 @@ class Version:
         latest_tag: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Bazaar tags.
@@ -1521,6 +1648,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Bazaar
@@ -1567,9 +1695,19 @@ class Version:
             if line.split()[1] != "?"
         }
         tags = [x[1] for x in sorted([(v, k) for k, v in tags_to_revs.items()], reverse=True)]
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, tags, latest_tag
-        )
+
+        matched_pattern = _match_version_pattern(pattern, tags, latest_tag, strict, pattern_prefix)
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                branch=branch,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
 
         distance = int(commit) - tags_to_revs[tag]
 
@@ -1596,6 +1734,7 @@ class Version:
         latest_tag: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Fossil tags.
@@ -1607,6 +1746,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Fossil
@@ -1690,9 +1830,22 @@ class Version:
             (line.rsplit(",", 1)[0][5:-1], int(line.rsplit(",", 1)[1]) - 1)
             for line in msg.splitlines()
         ]
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, [t for t, d in tags_to_distance], latest_tag
+
+        matched_pattern = _match_version_pattern(
+            pattern, [t for t, d in tags_to_distance], latest_tag, strict, pattern_prefix
         )
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                branch=branch,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
+
         distance = dict(tags_to_distance)[tag]
 
         version = cls(
@@ -1718,6 +1871,7 @@ class Version:
         latest_tag: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
     ) -> "Version":
         r"""
         Determine a version based on Pijul tags.
@@ -1730,6 +1884,7 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
         :returns: Detected version.
         """
         vcs = Vcs.Pijul
@@ -1819,9 +1974,19 @@ class Version:
             t["message"]
             for t in sorted(tag_meta_by_msg.values(), key=lambda x: x["timestamp"], reverse=True)
         ]
-        tag, base, stage, unmatched, tagged_metadata, epoch = _match_version_pattern(
-            pattern, tags, latest_tag
-        )
+
+        matched_pattern = _match_version_pattern(pattern, tags, latest_tag, strict, pattern_prefix)
+        if matched_pattern is None:
+            return cls._fallback(
+                strict,
+                distance=distance,
+                commit=commit,
+                dirty=dirty,
+                branch=branch,
+                timestamp=timestamp,
+                vcs=vcs,
+            )
+        tag, base, stage, unmatched, tagged_metadata, epoch = matched_pattern
 
         tag_id = tag_meta_by_msg[tag]["state"]
         _run_cmd("pijul tag checkout {}".format(tag_id), path, codes=[0, 1])
@@ -1859,6 +2024,8 @@ class Version:
         full_commit: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
+        ignore_untracked: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a detected version control system.
@@ -1887,13 +2054,26 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
+        :param ignore_untracked:
+            Ignore untracked files when determining whether the repository is dirty.
+            This is only used for Git currently.
         :returns: Detected version.
         """
         vcs = _detect_vcs_from_archival(path)
         if vcs is None:
             vcs = _detect_vcs(None, path)
         return cls._do_vcs_callback(
-            vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit, strict, path
+            vcs,
+            pattern,
+            latest_tag,
+            tag_dir,
+            tag_branch,
+            full_commit,
+            strict,
+            path,
+            pattern_prefix,
+            ignore_untracked,
         )
 
     @classmethod
@@ -1907,6 +2087,8 @@ class Version:
         full_commit: bool = False,
         strict: bool = False,
         path: Optional[Path] = None,
+        pattern_prefix: Optional[str] = None,
+        ignore_untracked: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a specific VCS setting.
@@ -1929,10 +2111,23 @@ class Version:
         :param strict: Elevate warnings to errors.
             When there are no tags, fail instead of falling back to 0.0.0.
         :param path: Directory to inspect, if not the current working directory.
+        :param pattern_prefix: Insert this after the pattern's start anchor (`^`).
+        :param ignore_untracked:
+            Ignore untracked files when determining whether the repository is dirty.
+            This is only used for Git currently.
         :returns: Detected version.
         """
         return cls._do_vcs_callback(
-            vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit, strict, path
+            vcs,
+            pattern,
+            latest_tag,
+            tag_dir,
+            tag_branch,
+            full_commit,
+            strict,
+            path,
+            pattern_prefix,
+            ignore_untracked,
         )
 
     @classmethod
@@ -1946,6 +2141,8 @@ class Version:
         full_commit: bool,
         strict: bool,
         path: Optional[Path],
+        pattern_prefix: Optional[str] = None,
+        ignore_untracked: bool = False,
     ) -> "Version":
         mapping = {
             Vcs.Any: cls.from_any_vcs,
@@ -1967,6 +2164,8 @@ class Version:
             ("full_commit", full_commit),
             ("strict", strict),
             ("path", path),
+            ("pattern_prefix", pattern_prefix),
+            ("ignore_untracked", ignore_untracked),
         ]:
             if kwarg in inspect.getfullargspec(callback).args:
                 kwargs[kwarg] = value
