@@ -1,6 +1,7 @@
 import datetime as dt
 import os
 import shutil
+import sys
 import time
 from contextlib import contextmanager
 from pathlib import Path
@@ -39,6 +40,17 @@ def is_git_legacy() -> bool:
     return version < [2, 7]
 
 
+def set_missing_env(key: str, value: str, alts: Optional[List[str]] = None) -> None:
+    if alts is None:
+        alts = []
+
+    for k in [key, *alts]:
+        if os.environ.get(k) is not None:
+            return
+
+    os.environ[key] = value
+
+
 def make_run_callback(where: Path) -> Callable:
     def inner(command, expected_code: int = 0, env: Optional[dict] = None):
         _, out = _run_cmd(command, where=where, codes=[expected_code], env=env)
@@ -112,6 +124,9 @@ def test__version__from_git__with_annotated_tags(tmp_path) -> None:
         # Detect dirty if untracked files
         (vcs / "bar.txt").write_text("bye")
         assert from_vcs() == Version("0.0.0", distance=1, dirty=True, branch=b)
+        assert from_vcs(ignore_untracked=True) == Version(
+            "0.0.0", distance=1, dirty=False, branch=b
+        )
 
         # Once the untracked file is removed we are no longer dirty
         (vcs / "bar.txt").unlink()
@@ -120,8 +135,9 @@ def test__version__from_git__with_annotated_tags(tmp_path) -> None:
         # Additional one-off check not in other VCS integration tests:
         # when the only tag in the repository does not match the pattern.
         run("git tag other -m Annotated")
+        assert from_vcs() == Version("0.0.0", dirty=False, branch=b)
         with pytest.raises(ValueError):
-            from_vcs()
+            from_vcs(strict=True)
 
         avoid_identical_ref_timestamps()
         run("git tag v0.1.0 -m Annotated")
@@ -154,6 +170,7 @@ def test__version__from_git__with_annotated_tags(tmp_path) -> None:
             # Verify tags with '/' work
             run("git tag test/v0.1.0")
             assert run(r'dunamai from any --pattern "^test/v(?P<base>\d\.\d\.\d)"') == "0.1.0"
+            assert run('dunamai from any --pattern-prefix "test/"') == "0.1.0"
 
         (vcs / "foo.txt").write_text("bye")
         assert from_vcs() == Version("0.1.0", dirty=True, branch=b)
@@ -479,6 +496,27 @@ def test__version__from_git__exclude_decoration(tmp_path) -> None:
         assert from_vcs() == Version("0.1.0", dirty=False, branch=b)
 
 
+# Older versions of Git fail with code 128:
+# "fatal: missing object 0000000000000000000000000000000000000000 for refs/tags/bad.txt"
+@pytest.mark.skipif(lacks_git_version([2, 7]), reason="Requires Git 2.7+")
+def test__version__from_git__broken_ref(tmp_path) -> None:
+    vcs = tmp_path / "dunamai-git-broken-ref"
+    vcs.mkdir()
+    run = make_run_callback(vcs)
+    from_vcs = make_from_callback(Version.from_git)
+    b = "master"
+
+    with chdir(vcs):
+        run("git init")
+        (vcs / "foo.txt").write_text("hi")
+        run("git add .")
+        run("git commit --no-gpg-sign -m Initial")
+        run("git tag v0.1.0 -m Release")
+        (vcs / ".git/refs/tags/bad.txt").touch()
+
+        assert from_vcs() == Version("0.1.0", dirty=False, branch=b)
+
+
 @pytest.mark.skipif(shutil.which("git") is None, reason="Requires Git")
 def test__version__not_a_repository(tmp_path) -> None:
     vcs = tmp_path / "dunamai-not-a-repo"
@@ -796,6 +834,10 @@ def test__version__from_fossil(tmp_path) -> None:
     from_vcs = make_from_callback(Version.from_fossil)
     b = "trunk"
 
+    if sys.platform != "win32":
+        set_missing_env("FOSSIL_HOME", str(REPO / "tests"), ["HOME", "XDG_CONFIG_HOME"])
+        set_missing_env("USER", "dunamai")
+
     with chdir(vcs):
         run("fossil init repo")
         run("fossil open repo --force")
