@@ -63,13 +63,19 @@ from_explicit_vcs = make_from_callback(Version.from_vcs)
 
 def test__pattern__regex() -> None:
     assert Pattern.Default.regex() == VERSION_SOURCE_PATTERN
-    assert Pattern.DefaultUnprefixed.regex() != VERSION_SOURCE_PATTERN
+    assert Pattern.DefaultUnprefixed.regex() == VERSION_SOURCE_PATTERN.replace("^v", "^v?", 1)
+    assert Pattern.Default.regex("foo-") == VERSION_SOURCE_PATTERN.replace("^", "^foo-", 1)
 
 
 def test__pattern__parse() -> None:
     assert Pattern.parse(r"(?P<base>\d+)") == r"(?P<base>\d+)"
+    assert Pattern.parse(r"(?P<base>\d+)", "foo-") == r"(?P<base>\d+)"
+    assert Pattern.parse(r"^(?P<base>\d+)", "foo-") == r"^foo-(?P<base>\d+)"
+
     assert Pattern.parse("default") == Pattern.Default.regex()
     assert Pattern.parse("default-unprefixed") == Pattern.DefaultUnprefixed.regex()
+    assert Pattern.parse("default", "foo-") == Pattern.Default.regex("foo-")
+
     with pytest.raises(ValueError):
         Pattern.parse(r"foo")
 
@@ -508,6 +514,14 @@ def test__version__bump() -> None:
     assert Version("1.2.3", stage=("a", None)).bump().serialize() == "1.2.3a2"
     assert Version("1.2.3", stage=("a", 4)).bump().serialize() == "1.2.3a5"
 
+    assert Version("1.2.3", distance=0).bump(smart=False).serialize() == "1.2.4"
+    assert Version("1.2.3", distance=0).bump(smart=True).serialize() == "1.2.3"
+    assert Version("1.2.3", distance=0).serialize(bump=True) == "1.2.3"
+
+    assert Version("1.2.3", distance=5).bump(smart=False).serialize() == "1.2.4.post5.dev0"
+    assert Version("1.2.3", distance=5).bump(smart=True).serialize() == "1.2.4.dev5"
+    assert Version("1.2.3", distance=5).serialize(bump=True) == "1.2.4.dev5"
+
 
 def test__version__parse():
     assert Version.parse("1.2.3") == Version("1.2.3")
